/**
Copyright (c) <2010> <Matthew Hausknecht, Peter Stone>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
**/

//-*-c++-*-
#ifndef INCLUDED_AlignGrab_h_
#define INCLUDED_AlignGrab_h_

#include "Behaviors/StateNode.h"
#include "Motion/MotionManager.h"
#include "Motion/MotionSequenceMC.h"
#include "Shared/mathutils.h"

using namespace mathutils;

class AlignGrab : public StateNode {

public:
  AlignGrab()
    : StateNode("AlignGrab","AlignGrab")
  {}

  AlignGrab(const std::string& nm)
    : StateNode("AlignGrab",nm)
  {}

protected:
  AlignGrab(const std::string &class_name, const std::string &node_name)
    : StateNode(class_name,node_name)
  {}
	
	
public:
  virtual void DoStart() {
    StateNode::DoStart(); // do this first (required)

    SharedObject<XLargeMotionSequenceMC> mseq_mc;
    erouter->addListener(this,EventBase::motmanEGID); //look for button input

    mseq_mc->advanceTime(800); // Legs out
    mseq_mc->LoadFile("algn1.pos");

    mseq_mc->advanceTime(800); // Push against wall
    mseq_mc->LoadFile("algn2.pos");

    mseq_mc->advanceTime(1300); // Back Legs Off
    mseq_mc->LoadFile("algn3.pos");

    mseq_mc->advanceTime(800); // Back Legs down 
    mseq_mc->LoadFile("algn4.pos");

    mseq_mc->advanceTime(700); // Sit on back legs
    mseq_mc->LoadFile("algn5.pos");

    mseq_mc->advanceTime(600); // Front legs off
    mseq_mc->LoadFile("algn6.pos");

    mseq_mc->advanceTime(600); // Grabbing pos
    mseq_mc->LoadFile("algn7.pos");
    
    motman->addPrunableMotion(mseq_mc);
  }

  virtual void processEvent(const EventBase& event) {
    if ((event.getGeneratorID()) == EventBase::motmanEGID && event.getTypeID() == EventBase::deactivateETID) {
      postCompletionEvent(1);
    }
  }

  virtual void DoStop() {
    erouter->removeListener(this); //generally a good idea, unsubscribe all
    StateNode::DoStop(); // do this last (required)
  }

  static std::string getClassDescription() { return "Block Shot Node"; }
  virtual std::string getDescription() const { return getClassDescription(); }


protected:

private:
  AlignGrab(const AlignGrab&); //!< don't call (copy constructor)
  AlignGrab& operator=(const AlignGrab&); //!< don't call (assignment operator)
};

#endif
