/*
Copyright (c) <2010> <Matthew Hausknecht, Peter Stone>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

//-*-c++-*-
#ifndef INCLUDED_BallTimer_h_
#define INCLUDED_BallTimer_h_

#include "Behaviors/StateNode.h"
#include "Motion/MotionManager.h"
#include "Motion/MotionSequenceMC.h"
#include "Shared/mathutils.h"
#include "Shared/Profiler.h"
#include "Events/EventRouter.h"
#include "Behaviors/BehaviorBase.h"
#include "Events/VisionObjectEvent.h"
#include "Shared/ProjectInterface.h"

#include "final/PolicyGradientAlgorithm.h"

using namespace mathutils;

class BallTimer : public StateNode {

public:
  BallTimer()
    : StateNode("BallTimer","BallTimer"), pg(NULL)
  {}

  BallTimer(const std::string& nm)
    : StateNode("BallTimer",nm), pg(NULL)
  {}

protected:
  BallTimer(const std::string &class_name, const std::string &node_name)
    : StateNode(class_name,node_name), pg(NULL)
  {}
	
	
public:
  virtual void DoStart() {
    StateNode::DoStart(); // do this first (required)

    t = Profiler::Timer::Timer();
    erouter->addListener(this,EventBase::visObjEGID,ProjectInterface::visOrangeSID);

  }

  void setPG (PolicyGradientAlgorithm *policyGrad) {
    pg = policyGrad;
  }

  virtual void processEvent(const EventBase& event) {
    if (event.getGeneratorID() == EventBase::visObjEGID) {
      const VisionObjectEvent &visev = *reinterpret_cast<const VisionObjectEvent*>(&event);

      if (visev.getObjectArea() < .1) return;
      long elapsed = t.elapsed().getMilliseconds(500); //Profiler::Timer::elapsed();

      std::cout << "Elapsed Time Since Kick: " << elapsed << "ms" << std::endl;
      pg->setCurrentPolicyScore(elapsed);
      postCompletionEvent(1);
    } 
  }

  virtual void DoStop() {
    erouter->removeListener(this); //generally a good idea, unsubscribe all
    StateNode::DoStop(); // do this last (required)
  }

  static std::string getClassDescription() { return "Variable Kick Node"; }
  virtual std::string getDescription() const { return getClassDescription(); }


protected:
  PolicyGradientAlgorithm *pg;
  Profiler::Timer t;

private:
  BallTimer(const BallTimer&); //!< don't call (copy constructor)
  BallTimer& operator=(const BallTimer&); //!< don't call (assignment operator)
};

#endif
