/*
Copyright (c) <2010> <Matthew Hausknecht, Peter Stone>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef INCLUDED_KickLearner_h_
#define INCLUDED_KickLearner_h_

#include "Behaviors/StateNode.h"
#include "Behaviors/Nodes/WalkNode.h"
#include "Motion/MotionManager.h"
#include "Behaviors/Transitions/CompletionTrans.h"
#include "Behaviors/Transitions/TimeOutTrans.h"
#include "Behaviors/Transitions/NullTrans.h"
#include "final/VariableKick.h"
#include "final/MyGrab.h"
#include "final/BallTimer.h"
#include "final/TouchTimer.h"
#include "final/MessageTimer.h"
#include "final/AlignGrab.h"
#include "final/PolicyGradientAlgorithm.h"

#include "Shared/RobotInfo.h"

class KickLearner : public StateNode {
 public:
 KickLearner():StateNode("KickLearner","KickLearner"), start(NULL),
    pg(NULL)
    {}

 KickLearner(const std::string& nm):StateNode("KickLearner",nm), start(NULL),
    pg(NULL)
    {}

  ~KickLearner() {
    if(issetup)
      teardown();
  }

  void setup(){
    // All Joints & Timings for each posture -- minus tail joints
    // starts learning from the power kick
    /*double initialPolicy[] = {24,-40,0,-20,-50,0,0,60,  -8, 30,  60,  -8, 30,  -45, 6,  75,  -45, 6,  75,
			      24,0,-75,-20,0,0,0,  -10, -8, 30,  -10, -8, 30,  -60, -5, 120, -60, -5, 120,
			      16,0,-75,-20,0,0,0,  -10, -8, 30,  -10, -8, 30,  -60, 0,  120, -60, 0,  120,
			      16,-35,35,-20,0,0,0, 75,  -7, 15,  75,  -7, 15,  30,  0,  120, 30,  0,  120,
			      24,-35,35,-20,0,0,0, 75,  -7, 15,  75,  -7, 15,  30,  0,  120, 30,  0,  120,
			      48,0,0,-20,0,0,0,    -5,  10, 115, -5,  10, 115, -30, 6,  104, -30, 6,  104};
    */

    // Symmetric parametrization initial policy (Power Kick):
    //                          frame HT  HP   HN   Mou  FR    FE  FKn  BRo  BE  BKn
    /*double initialPolicy[] = { 24, -40,   0, -20, -50,  60,  -8,  30, -45,  6,   75,
			         24,   0, -75, -20,   0, -10,  -8,  30, -60, -5,  120,
			         16,   0, -75, -20,   0, -10,  -8,  30, -60,  0,  120,
			         16, -35,  35, -20,   0,  75,  -7,  15,  30,  0,  120,
			         24, -35,  35, -20,   0,  75,  -7,  15,  30,  0,  120,
			         48,   0,   0, -20,   0,  -5,  10, 115, -30,  6,  104};*/

    // Best hill Climbing Kick Symmetric Parameterization
    /*double initialPolicy[] = {24.344, -47.4839, -3.70496, -4.69269, -53.8449, 83.0811, 10.4333, 54.5972, -48.375, -6.87819, 77.0718,
			      26.1434, -18.8483, -88, -1.28784, -3.70408, 20.9843, 4.46749, 32.0497, -64.6041, 7.11151, 118.334,
			      24.5257, -0.663021, -87.0323, -7.88955, -9.42374, -3.41968, 5.16411, -12.6524, -41.1155, 1.34013, 105.869,
			      14.3667, -37.9729, 55.6004, -3.52831, -23.7132, 72.3329, 3.24982, -11.8275, 56.7936, 0.0308476, 122,
			      1.31957, -29.2837, 54.2799, -15, -15.0727, 108.705, -4.97844, 0.0163741, 47.9327, 4.41657, 98.4882,
			      29.6368, -12.4431, 5.70258, -15, -15.7567, -17.248, 5.12743, 111.741, -68.3712, -10, 108.891};*/

    // Best Hill Climb Kick Symmetric #2
    double initialPolicy[] = {42.9101, -27.0284, 4.14017, -4.70668, -55, 76.982, -4.79606, 64.8182, -46.3681, -10, 65.4155, 23.1366, -5.41879, -69.377, 6.66947, -5.80628, -24.2091, 8.78205, 36.7724, -74.13, 25.7255, 122, 19.8764, -4.3428, -88, -9.37422, -12.4835, -10.4283, -1.13077, 4.59844, -44.4941, -1.35682, 91.0595, 26.0112, -46.0197, 57.2966, 4.76001, -20.3004, 78.4637, 10.3961, -25, 95.3476, 21.0577, 122, 16.7407, -38.1342, 27.4011, -15, -21.3494, 128.533, -2.24312, 11.4384, 19.3939, 24.9263, 107.177, 35.116, 0, 40.0046, -15, -15.2574, 7.32282, 5.47558, 122, -43.4153, 4.43759, 122};
			      
    

    // Symmetric Nothing Parameterization -- Good start point for a nose hit
    /*double initialPolicy[] = { 25, -61, 0, 50, -3, 33, -4, 91, -43, 0, 127,
			       25, -80, 0, 44, -6, 11, 0,  49, -34, 0, 126,
			       25,   0, 0,  0,-35, 16, 0,  37,  -8, 0, 121,
			       25, -40, 0,-11,  0, 44, 0,  66, -21, 0, 126,
			       25, -50, 0,-11,  0, 50, 0,  63, -22, 0, 125,
			       25, -63, 0,-11,  0, 67, 0,  62, -22, 0, 125};*/

    // Symmetric chest push.... Really sucks
    /*double initialPolicy[] = { 60, 0, 0, -20, 0, -5, 10, 115, -30, 6, 104,
			       40, 0, 0,   0, 0, -119, -4, 124, 71, 0, 43,
			       20, 0, 0,   0, 0, -50, -2, 124, 71, 0, 43,
			       20, 0, 0,   0, 0, -13, 0, 124, 71, 0, 43,
			       20, 0, 0,   0, 0, 0, 0, 124, 71, 0, 43,
			       40, 0, 0,   0, 0, 20, 0, 124, 71, 0, 43};*/


    // Kick timing initial Parameterization: Very Slow
    //double initialPolicy[] = {75.0,75.0,75.0,75.0,75.0,75.0};
    
    // Best Kick found by PG Algorithm
    /*double initialPolicy[] = { 23.1224, -49.2517, -5.98653, -4.27761, -52.6633, 85.6147, 8.98929, 51.0088, -54.4447, -5.80674, 80.107, 25.1688, -18.6368, -84.5136, -2.96202, -11.5002, 21.0803, 4.28637, 34.4619, -63.9083, 6.01989, 122, 21.8809, -9.66904, -88, -6.9221, -9.36704, -2.15887, 2.27538, -6.94946, -39.3566, -4.76196, 105.807, 14.0877, -30.5635, 54.5137, -5.10134, -20.1661, 72.0425, 3.286, -13.2529, 55.8711, 1.2701, 122, 1.80712, -27.6887, 53.2648, -10.4123, -16.4368, 111.206, -5.40436, -2.74324, 48.3031, 8.47706, 102.93, 37.3671, -12.9685, 11.6231, -12.7972, -19.0221, -16.8977, 4.673, 119.342, -65.5067, -6.79058, 102.866};*/

    // 2nd Best PG Kick
    //double initialPolicy[] = {21.3858, -40.9115, -8.23759, -12.038, -46.3943, 107.444, 6.50508, 44.821, -39.9574, -3.86025, 84.911, 35.0917, 0, -70.6432, -9.01993, -10.9566, 12.0329, -4.41246, 13.2995, -38.7424, -0.524871, 110.319, 16.5572, -13.2977, -77.0854, -8.49565, -14.1244, 21.9738, -9.43808, 5.61579, -21.7184, -0.623487, 90.1803, 11.7345, -31.8858, 31.9048, 1.1039, -20.0735, 67.34, 7.63786, -23.1077, 26.2733, -3.03609, 106.401, 4.06047, -19.9866, 50.4819, -11.3016, -6.1436, 114.742, -9.08205, 7.48515, -58.3739, -5.21847, 98.3416, 25.5439, -11.2588, 0.471107, -9.33782, -16.4086, -40.3123, -3.93877, 105.566, -31.8776, -2.54945, 107.718};


    pg = new PolicyGradientAlgorithm(initialPolicy);

    MyGrab *my_grab = new MyGrab();
    VariableKick *varKick = new VariableKick();
    MessageTimer *messageTimer = new MessageTimer();

    varKick->setPG(pg);
    messageTimer->setPG(pg);

    addNode(my_grab);
    addNode(varKick);
    addNode(messageTimer);

    start = my_grab;


    // Transition From Grab to Kick
    my_grab->addTransition(new CompletionTrans(varKick,0));

    // Transition from kick to touchTimer
    varKick->addTransition(new CompletionTrans(messageTimer,0));

    // Transition from touchTimer to grab
    messageTimer->addTransition(new CompletionTrans(varKick,0));


    StateNode::setup();
  };

  void DoStart(){
    StateNode::DoStart();
    start->DoStart();
  };

  void DoStop(){
    erouter->removeListener(this);
    StateNode::DoStop();
  };
  
  void teardown(){
    StateNode::teardown();
  };

 protected:
  StateNode * start; //!< the node to begin within on DoStart() (turn)

  PolicyGradientAlgorithm *pg;

 private:
  KickLearner(const KickLearner&); //!< don't use
  KickLearner operator=(const KickLearner&); //!< don't use
};

#endif
