/*
Copyright (c) <2010> <Matthew Hausknecht, Peter Stone>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

//-*-c++-*-
#ifndef INCLUDED_LearnedKick_h_
#define INCLUDED_LearnedKick_h_

#include "Behaviors/StateNode.h"
#include "Motion/MotionManager.h"
#include "Motion/MotionSequenceMC.h"
#include "Shared/mathutils.h"
#include "final/PolicyGradientAlgorithm.h"

using namespace mathutils;

class LearnedKick : public StateNode {

public:
  LearnedKick()
    : StateNode("LearnedKick","LearnedKick"), pg(NULL)
  {}

  LearnedKick(const std::string& nm)
    : StateNode("LearnedKick",nm), pg(NULL)
  {}

protected:
  LearnedKick(const std::string &class_name, const std::string &node_name)
    : StateNode(class_name,node_name), pg(NULL)
  {}
	
	
public:
  virtual void DoStart() {
    StateNode::DoStart(); // do this first (required)

    SharedObject<MediumMotionSequenceMC> mseq_mc;
    erouter->addListener(this,EventBase::motmanEGID); //look for button input

    // Best Learned Kick!
    setPose(mseq_mc,42, -27, 4, -4, -55, 0,0,  76, -4, 64,76, -4, 64, -46, -10, 65,-46, -10, 65);
    setPose(mseq_mc,23, -5, -69, 6, -5,0,0,   -24, 8, 36,-24, 8, 36, -74, 25, 122,-74, 25, 122);
    setPose(mseq_mc,19, -4, -88, -9, -12,0,0, -10, -1, 4,-10, -1, 4, -44, -1, 91,-44, -1, 91);
    setPose(mseq_mc,26, -46, 57, 4, -20,0,0,   78, 10, -25,78, 10, -25, 95, 21, 122,95, 21, 122);
    setPose(mseq_mc,16, -38, 27, -15, -21,0,0, 128, -2, 11,128, -2, 11, 19, 24, 107,19, 24, 107);
    setPose(mseq_mc,35, 0, 40, -15, -15,0,0,   7, 5, 122,7, 5, 122, -43, 4, 122,7, 5, 122);

    
    // Symmetry parameterization with timing adjustments
    /*double *p = new double[66];
    pg->getCurrentPolicy(p);
    setPose(mseq_mc,(int)p[0],(int)p[1],(int)p[2],(int)p[3],(int)p[4],0,0,(int)p[5],(int)p[6],(int)p[7],(int)p[5],(int)p[6],(int)p[7],(int)p[8],(int)p[9],(int)p[10],(int)p[8],(int)p[9],(int)p[10]);
    setPose(mseq_mc,(int)p[11],(int)p[12],(int)p[13],(int)p[14],(int)p[15],0,0,(int)p[16],(int)p[17],(int)p[18],(int)p[16],(int)p[17],(int)p[18],(int)p[19],(int)p[20],(int)p[21],(int)p[19],(int)p[20],(int)p[21]);
    setPose(mseq_mc,(int)p[22],(int)p[23],(int)p[24],(int)p[25],(int)p[26],0,0,(int)p[27],(int)p[28],(int)p[29],(int)p[27],(int)p[28],(int)p[29],(int)p[30],(int)p[31],(int)p[32],(int)p[30],(int)p[31],(int)p[32]);
    setPose(mseq_mc,(int)p[33],(int)p[34],(int)p[35],(int)p[36],(int)p[37],0,0,(int)p[38],(int)p[39],(int)p[40],(int)p[38],(int)p[39],(int)p[40],(int)p[41],(int)p[42],(int)p[43],(int)p[41],(int)p[42],(int)p[43]);
    setPose(mseq_mc,(int)p[44],(int)p[45],(int)p[46],(int)p[47],(int)p[48],0,0,(int)p[49],(int)p[50],(int)p[51],(int)p[49],(int)p[50],(int)p[51],(int)p[52],(int)p[53],(int)p[54],(int)p[52],(int)p[53],(int)p[54]);
    setPose(mseq_mc,(int)p[55],(int)p[56],(int)p[57],(int)p[58],(int)p[59],0,0,(int)p[60],(int)p[61],(int)p[62],(int)p[60],(int)p[61],(int)p[62],(int)p[63],(int)p[64],(int)p[65],(int)p[63],(int)p[64],(int)p[65]);*/
    
    motman->addPrunableMotion(mseq_mc);
  }

  /* Attempts to modulate the power of a kick in order to make the ball
   * go the desired distance in centimeters.
   */
  void variableKick (SharedObject<MediumMotionSequenceMC> mseq_mc, int desiredDistCM) {
    //Quadratic: -.33x^2 + 17.7174x + 160.0987
    //Max @ x=26.84 with value 397.91
    if (desiredDistCM < 0)
      return;
    
    int numFrames;
    if (desiredDistCM >= 398) { // Give hardest known kick
      numFrames = (int)round(26.84);
    } else if (desiredDistCM >= 160) { // Use Left Side of Quadratic
      numFrames = (int) round(-(sqrt(15.0)*sqrt(87437.0-220.0*desiredDistCM)-885.0)/33.0);
    } else { // Use Right Side of Quadratic
      numFrames = (int) round((sqrt(15.0)*sqrt(87437.0-220.0*desiredDistCM)+885.0)/33.0);
    }

    std::cout << "Using " << numFrames << " Frames for kick" << std::endl;
    
    setPose(mseq_mc,24,-40,0,-20,-50,0,0,60,-8,30,60,-8,30,-45,6,75,-45,6,75);
    setPose(mseq_mc,24,0,-75,-20,0,0,0,-10,-8,30,-10,-8,30,-60,-5,120,-60,-5,120);
    setPose(mseq_mc,16,0,-75,-20,0,0,0,-10,-8,30,-10,-8,30,-60,0,120,-60,0,120);
    setPose(mseq_mc,numFrames,-35,35,-20,0,0,0,75,-7,15,75,-7,15,30,0,120,30,0,120);
    setPose(mseq_mc,24,-35,35,-20,0,0,0,75,-7,15,75,-7,15,30,0,120,30,0,120);
    setPose(mseq_mc,48,0,0,-20,0,0,0,-5,10,115,-5,10,115,-30,6,104,-30,6,104);
  }

  void setPose (SharedObject<MediumMotionSequenceMC> mseq_mc, int numFrames,
		int neckTilt, int headPan, int headTilt,
		int mouth, int tailTilt, int tailPan,
		int frontLeftRotator, int frontLeftAbductor, int frontLeftKnee,
		int frontRightRotator, int frontRightAbductor, int frontRightKnee,
		int backLeftRotator, int backLeftAbductor, int backLeftKnee,
		int backRightRotator, int backRightAbductor, int backRightKnee) {
    // Number of frames to get into this pose
    mseq_mc->advanceTime(8*numFrames);
    
    // Head
    mseq_mc->setOutputCmd(HeadOffset+TiltOffset, RAD(neckTilt));
    mseq_mc->setOutputCmd(HeadOffset+PanOffset, RAD(headPan));
    mseq_mc->setOutputCmd(HeadOffset+NodOffset, RAD(headTilt));

    // Mouth & Tail
    mseq_mc->setOutputCmd(MouthOffset, RAD(mouth));
    mseq_mc->setOutputCmd(TailOffset+TiltOffset, RAD(tailTilt));
    mseq_mc->setOutputCmd(TailOffset+PanOffset, RAD(tailPan));

    // Legs
    mseq_mc->setOutputCmd(LFrLegOffset+RotatorOffset, RAD(frontLeftRotator));
    mseq_mc->setOutputCmd(LFrLegOffset+ElevatorOffset, RAD(frontLeftAbductor));
    mseq_mc->setOutputCmd(LFrLegOffset+KneeOffset, RAD(frontLeftKnee));
    
    mseq_mc->setOutputCmd(RFrLegOffset+RotatorOffset, RAD(frontRightRotator));
    mseq_mc->setOutputCmd(RFrLegOffset+ElevatorOffset, RAD(frontRightAbductor));
    mseq_mc->setOutputCmd(RFrLegOffset+KneeOffset, RAD(frontRightKnee));

    mseq_mc->setOutputCmd(LBkLegOffset+RotatorOffset, RAD(backLeftRotator));
    mseq_mc->setOutputCmd(LBkLegOffset+ElevatorOffset, RAD(backLeftAbductor));
    mseq_mc->setOutputCmd(LBkLegOffset+KneeOffset, RAD(backLeftKnee));

    mseq_mc->setOutputCmd(RBkLegOffset+RotatorOffset, RAD(backRightRotator));
    mseq_mc->setOutputCmd(RBkLegOffset+ElevatorOffset, RAD(backRightAbductor));
    mseq_mc->setOutputCmd(RBkLegOffset+KneeOffset, RAD(backRightKnee));
  }

  void setPG (PolicyGradientAlgorithm *policyGrad) {
    pg = policyGrad;
  }

  virtual void processEvent(const EventBase& event) {
    if ((event.getGeneratorID()) == EventBase::motmanEGID && event.getTypeID() == EventBase::deactivateETID) {
      postCompletionEvent(0);
    }
  }

  virtual void DoStop() {
    erouter->removeListener(this); //generally a good idea, unsubscribe all
    StateNode::DoStop(); // do this last (required)
  }

  static std::string getClassDescription() { return "Variable Kick Node"; }
  virtual std::string getDescription() const { return getClassDescription(); }


protected:
  PolicyGradientAlgorithm *pg;

private:
  LearnedKick(const LearnedKick&); //!< don't call (copy constructor)
  LearnedKick& operator=(const LearnedKick&); //!< don't call (assignment operator)
};

#endif
