#ifndef INCLUDED_LearnedKickDemo_h_
#define INCLUDED_LearnedKickDemo_h_

#include "Behaviors/StateNode.h"
#include "Behaviors/Nodes/WalkNode.h"
#include "Motion/MotionManager.h"
#include "Behaviors/Transitions/CompletionTrans.h"
#include "Behaviors/Transitions/TimeOutTrans.h"
#include "Behaviors/Transitions/NullTrans.h"
/*
Copyright (c) <2010> <Matthew Hausknecht, Peter Stone>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#include "final/VariableKick.h"
#include "final/LearnedKick.h"
#include "final/MyGrab.h"
#include "final/BallTimer.h"
#include "final/TouchTimer.h"
#include "final/MessageTimer.h"
#include "final/AlignGrab.h"
#include "final/PolicyGradientAlgorithm.h"

#include "Shared/RobotInfo.h"

class LearnedKickDemo : public StateNode {
 public:
 LearnedKickDemo():StateNode("LearnedKickDemo","LearnedKickDemo"), start(NULL),
    pg(NULL)
    {}

 LearnedKickDemo(const std::string& nm):StateNode("LearnedKickDemo",nm), start(NULL),
    pg(NULL)
    {}

  ~LearnedKickDemo() {
    if(issetup)
      teardown();
  }

  void setup(){
    double initialPolicy[] = {42.9101, -27.0284, 4.14017, -4.70668, -55, 76.982, -4.79606, 64.8182, -46.3681, -10, 65.4155, 23.1366, -5.41879, -69.377, 6.66947, -5.80628, -24.2091, 8.78205, 36.7724, -74.13, 25.7255, 122, 19.8764, -4.3428, -88, -9.37422, -12.4835, -10.4283, -1.13077, 4.59844, -44.4941, -1.35682, 91.0595, 26.0112, -46.0197, 57.2966, 4.76001, -20.3004, 78.4637, 10.3961, -25, 95.3476, 21.0577, 122, 16.7407, -38.1342, 27.4011, -15, -21.3494, 128.533, -2.24312, 11.4384, 19.3939, 24.9263, 107.177, 35.116, 0, 40.0046, -15, -15.2574, 7.32282, 5.47558, 122, -43.4153, 4.43759, 122};

    pg = new PolicyGradientAlgorithm(initialPolicy);
    
    MyGrab *my_grab = new MyGrab();
    LearnedKick *learnedKick = new LearnedKick();
    TouchTimer *touchTimer = new TouchTimer();

    learnedKick->setPG(pg);

    addNode(my_grab);
    addNode(learnedKick);
    addNode(touchTimer);

    start = my_grab;


    // Transition From Grab to Kick
    my_grab->addTransition(new CompletionTrans(learnedKick,0));

    // Transition from kick to touchTimer
    learnedKick->addTransition(new CompletionTrans(touchTimer,0));

    // Transition from touchTimer to grab
    touchTimer->addTransition(new CompletionTrans(my_grab,0));


    StateNode::setup();
  };

  void DoStart(){
    StateNode::DoStart();
    start->DoStart();
  };

  void DoStop(){
    erouter->removeListener(this);
    StateNode::DoStop();
  };
  
  void teardown(){
    StateNode::teardown();
  };

 protected:
  StateNode * start; //!< the node to begin within on DoStart() (turn)

  PolicyGradientAlgorithm *pg;

 private:
  LearnedKickDemo(const LearnedKickDemo&); //!< don't use
  LearnedKickDemo operator=(const LearnedKickDemo&); //!< don't use
};

#endif
