/*
Copyright (c) <2010> <Matthew Hausknecht, Peter Stone>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

//-*-c++-*-
#ifndef INCLUDED_MessageTimer_h_
#define INCLUDED_MessageTimer_h_

#include "Behaviors/StateNode.h"
#include "Motion/MotionManager.h"
#include "Motion/MotionSequenceMC.h"
#include "Shared/mathutils.h"
#include "Shared/Profiler.h"
#include "Events/EventRouter.h"
#include "Behaviors/BehaviorBase.h"
#include "Events/VisionObjectEvent.h"
#include "Shared/ProjectInterface.h"
#include "Events/TextMsgEvent.h"

#include "final/PolicyGradientAlgorithm.h"

using namespace mathutils;

class MessageTimer : public StateNode {

public:
  MessageTimer()
    : StateNode("MessageTimer","MessageTimer"), pg(NULL)
  {}

  MessageTimer(const std::string& nm)
    : StateNode("MessageTimer",nm), pg(NULL)
  {}

protected:
  MessageTimer(const std::string &class_name, const std::string &node_name)
    : StateNode(class_name,node_name), pg(NULL)
  {}
	
	
public:
  virtual void DoStart() {
    StateNode::DoStart(); // do this first (required)

    erouter->addListener(this,EventBase::textmsgEGID);

    
    SharedObject<MediumMotionSequenceMC> mseq_mc;
    erouter->addListener(this,EventBase::motmanEGID); //look for button input

    mseq_mc->advanceTime(1000); // Capture
    mseq_mc->LoadFile("capt4.pos");
    
    motman->addPrunableMotion(mseq_mc);
  }

  void setPG (PolicyGradientAlgorithm *policyGrad) {
    pg = policyGrad;
  }

  virtual void processEvent(const EventBase& event) {
    if (event.getGeneratorID() == EventBase::textmsgEGID) {

      const TextMsgEvent &txtev = dynamic_cast<const TextMsgEvent&>(event);

      int result = atoi(txtev.getText().c_str());
      std::cout << "Result: " << result << std::endl;

      //pg->setCurrentPolicyScore(result);
      postCompletionEvent(1);
    } 
  }

  virtual void DoStop() {
    erouter->removeListener(this); //generally a good idea, unsubscribe all
    StateNode::DoStop(); // do this last (required)
  }

  static std::string getClassDescription() { return "Variable Kick Node"; }
  virtual std::string getDescription() const { return getClassDescription(); }


protected:
  PolicyGradientAlgorithm *pg;

private:
  MessageTimer(const MessageTimer&); //!< don't call (copy constructor)
  MessageTimer& operator=(const MessageTimer&); //!< don't call (assignment operator)
};

#endif
