/*
Copyright (c) <2010> <Nate Kohl>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef _POLICY_GRADIENT_ALGORITHM_H
#define _POLICY_GRADIENT_ALGORITHM_H

#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <math.h>



using namespace std;


class PolicyGradientAlgorithm {
 public:
  // chin pinch
  static const int NUM_DIMENSIONS = 66;
  static const int NUM_POLICIES_PER_ITERATION = 5;
  static const int NUM_TRIALS_PER_POLICY = 2;
  
  // walking
  //static const int NUM_DIMENSIONS = 14;
  //static const int NUM_POLICIES_PER_ITERATION = 15;
  //static const int NUM_TRIALS_PER_POLICY = 1;
  static const double STEP_SIZE = 2.0;
  static const bool USE_ZERO_DELTAS = true;

  // create a new policy gradient algorithm
  // chin pinch
  PolicyGradientAlgorithm( double initialPolicy[] );

  // walking
  //PolicyGradientAlgorithm( GaitParams gp );

  // get the current policy that needs to be evaluated
  // chin pinch
  void getCurrentPolicy( double* result );

  // walking
  //void getCurrentPolicy( GaitParams* gp );

  // set the score for the current policy (recomputes policies if all
  // scores have been recieved)
  void setCurrentPolicyScore( double score );

  void printCurrentPolicy();

 private:
  static const double DELTAS[];
  static const double MINVALS[];
  static const double MAXVALS[];
  static const char LOGFILE[];
  static const char ALGFILE[];

  // details of the algorithm should go to this file
  ofstream algout;

  // the index of the policy that we're currently evaluating
  int currentPolicyIdx;
  // a running sum of trial results and a counter of how many trials
  // we've performed for this policy
  double currentTrialSum;
  int currentTrialCount;

  double policies[NUM_POLICIES_PER_ITERATION][NUM_DIMENSIONS];
  double results[NUM_POLICIES_PER_ITERATION];
  
  void resetAlgorithm();
  double frand( double min, double max );
  double randomDelta( int dimension );
  void computeNeighboringPolicies();
  void checkMinsAndMaxes();

  // These two functions both do the same job: the first one computes
  // the best direction to hill-climb in by estimating the gradient of
  // the objective function, and the second one simply moves the
  // current policy to the position of the highest-scoring policy each
  // iteration.  We should use one or the other for any given run.
  void computeBestDirection();
  void computeBestPolicy();

};

#endif //_POLICY_GRADIENT_ALGORITHM_H
