/*
Copyright (c) <2010> <Matthew Hausknecht, Peter Stone>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef INCLUDED_PowerKickDemo_h_
#define INCLUDED_PowerKickDemo_h_

#include "Behaviors/StateNode.h"
#include "Behaviors/Nodes/WalkNode.h"
#include "Motion/MotionManager.h"
#include "Behaviors/Transitions/CompletionTrans.h"
#include "Behaviors/Transitions/TimeOutTrans.h"
#include "Behaviors/Transitions/NullTrans.h"
#include "final/VariableKick.h"
#include "final/PowerKick.h"
#include "final/MyGrab.h"
#include "final/BallTimer.h"
#include "final/TouchTimer.h"
#include "final/MessageTimer.h"
#include "final/AlignGrab.h"
#include "final/PolicyGradientAlgorithm.h"

#include "Shared/RobotInfo.h"

class PowerKickDemo : public StateNode {
 public:
 PowerKickDemo():StateNode("PowerKickDemo","PowerKickDemo"), start(NULL),
    pg(NULL)
    {}

 PowerKickDemo(const std::string& nm):StateNode("PowerKickDemo",nm), start(NULL),
    pg(NULL)
    {}

  ~PowerKickDemo() {
    if(issetup)
      teardown();
  }

  void setup(){
    MyGrab *my_grab = new MyGrab();
    PowerKick *powerKick = new PowerKick();
    TouchTimer *touchTimer = new TouchTimer();

    addNode(my_grab);
    addNode(powerKick);
    addNode(touchTimer);

    start = my_grab;


    // Transition From Grab to Kick
    my_grab->addTransition(new CompletionTrans(powerKick,0));

    // Transition from kick to touchTimer
    powerKick->addTransition(new CompletionTrans(touchTimer,0));

    // Transition from touchTimer to grab
    touchTimer->addTransition(new CompletionTrans(my_grab,0));


    StateNode::setup();
  };

  void DoStart(){
    StateNode::DoStart();
    start->DoStart();
  };

  void DoStop(){
    erouter->removeListener(this);
    StateNode::DoStop();
  };
  
  void teardown(){
    StateNode::teardown();
  };

 protected:
  StateNode * start; //!< the node to begin within on DoStart() (turn)

  PolicyGradientAlgorithm *pg;

 private:
  PowerKickDemo(const PowerKickDemo&); //!< don't use
  PowerKickDemo operator=(const PowerKickDemo&); //!< don't use
};

#endif
